import re

import requests

from .exceptions import QujingInvokeError


class QujingByHttp:
    def __init__(self, host, port_config=61000, protocol="http"):
        self.host = host
        self.port_config = port_config
        self.protocol = protocol
        self.base_config_url = f"{self.protocol}://{self.host}:{self.port_config}"
        self.base_invoke_url = f"{self.protocol}://{self.host}:$port/invoke"

    def get_pid(self, packages: list, **kwargs):
        # 获取包名对应的进程号，即端口号
        url = f"{self.base_config_url}/manualguid"
        kwargs.setdefault("timeout", 5)
        resp = requests.get(url, **kwargs)
        ports = {package: re.findall(f"<td>{package}</td>.*?<td>(.*?)</td>", resp.text, re.S) for package in packages}
        ports = {package: int(port[0]) for package, port in ports.items() if len(port)}
        return ports

    def set_app(self, packages: list, **kwargs):
        # 设置目标app，设置后才能通过invoke接口调用
        url = f"{self.base_config_url}/settargetapp"
        params = {i: i for i in packages}
        kwargs.setdefault("timeout", 5)
        resp = requests.get(url, params=params, **kwargs)
        return resp.status_code == 200

    def invoke(self, port_invoke: int, data: dict, **kwargs):
        # 调用目标app内部函数的接口
        url = self.base_invoke_url.replace("$port", str(port_invoke))
        kwargs.setdefault("timeout", 5)
        try:
            resp = requests.post(url, data=data, **kwargs)
        except Exception as e:
            raise QujingInvokeError({"code": 500, "data": data, "error": str(e), "type": "error"})
        # 处理返回的数据
        _text = resp.text.strip()
        if _text.startswith("Base64#"):
            datatype = "base64"
            resp_data = _text[7:]
        elif _text.startswith("raw#"):
            datatype = "raw"
            resp_data = _text[4:]
        else:
            datatype = "raw"
            resp_data = _text
        return {"data": resp_data, "dtype": datatype}
